@@ -16,6 +16,8 @@ You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 """
 
+import uuid
+
 from gillespy2.core.gillespyError import *
 from gillespy2.core.jsonify import Jsonify
 
@@ -39,7 +41,12 @@ class EventAssignment(Jsonify):
 
     """
 
-    def __init__(self, variable=None, expression=None):
+    def __init__(self, name=None, variable=None, expression=None):
+
+        if name in (None, ""):
+            self.name = f'evn{uuid.uuid4()}'.replace('-', '_')
+        else:
+            self.name = name
 
         self.variable = variable
         self.expression = expression
