@@ -59,4 +59,24 @@ class GillesPySolver:
 
     def get_solver_settings(self):
 
-        raise SimulationError("This abstract solver class cannot be used directly")
\ No newline at end of file
+        raise SimulationError("This abstract solver class cannot be used directly")
+
+    def get_increment(self, increment):
+        """
+        Set the default increment value if it was not provided
+
+        :param increment: The current value of increment.
+        :type increment: int
+        """
+        if increment is None:
+            return self.model.tspan[-1] - self.model.tspan[-2]
+        if self.model.user_set_tspan:
+            raise  SimulationError(
+                """
+                Failed while preparing to run the model. Both increment and timespan are set.
+
+                To continue either remove your `timespan` definition from your Model or remove the 
+                `increment` argument from this `solver.run()` call.               
+                """
+            )
+        return increment
\ No newline at end of file
