@@ -180,6 +180,7 @@ class Model(SortableObject, Jsonify):
 
         if tspan is None:
             self.timespan(np.linspace(0, 20, 401))
+            self.user_set_tspan = False
         else:
             self.timespan(tspan)
 
@@ -415,14 +416,16 @@ class Model(SortableObject, Jsonify):
             for p in sorted(params):
                 self.add_parameter(p)
         else:
-            try:
+            if isinstance(params, Parameter) or type(params).__name__ == 'Parameter':
                 problem = self.problem_with_name(params.name)
                 if problem is not None:
                     raise problem
+                self.update_namespace()
+                params._evaluate(self.namespace)
                 self.listOfParameters[params.name] = params
                 self._listOfParameters[params.name] = 'P{}'.format(len(self._listOfParameters))
-            except Exception as e:
-                raise ParameterError("Error using {} as a Parameter. Reason given: {}".format(params, e))
+            else:
+                raise ParameterError("Parameter {}  must be of type {}, it is of type {}".format(params, str(type(Parameter)), str(params) ))
         return params
 
     def delete_parameter(self, obj):
@@ -449,7 +452,7 @@ class Model(SortableObject, Jsonify):
 
         p = self.listOfParameters[p_name]
         p.expression = expression
-        p.evaluate()
+        p._evaluate()
 
     def resolve_parameters(self):
         """ Internal function:
@@ -458,10 +461,7 @@ class Model(SortableObject, Jsonify):
         """
         self.update_namespace()
         for param in self.listOfParameters:
-            try:
-                self.listOfParameters[param].evaluate(self.namespace)
-            except:
-                raise ParameterError("Could not resolve Parameter expression {} to a scalar value.".format(param))
+            self.listOfParameters[param]._evaluate(self.namespace)
 
     def delete_all_parameters(self):
         """ Deletes all parameters from model. """
@@ -672,6 +672,7 @@ class Model(SortableObject, Jsonify):
         isuniform = np.isclose(other_diff, first_diff).all()
 
         if isuniform:
+            self.user_set_tspan = True
             self.tspan = time_span
         else:
             raise InvalidModelError("StochKit only supports uniform timespans")
@@ -963,6 +964,7 @@ class Model(SortableObject, Jsonify):
             from gillespy2.core import log
             log.warning('show_labels = False is deprecated. Future releases '
                         'of GillesPy2 may not support this feature.')
+
         if t is None:
             t = self.tspan[-1]
 
@@ -971,10 +973,19 @@ class Model(SortableObject, Jsonify):
                 solver = self.get_best_solver_algo(algorithm)
             else:
                 solver = self.get_best_solver()
-        if increment is None:
-            increment = self.tspan[-1] - self.tspan[-2]
+
+        if self.user_set_tspan and increment is not None:
+            raise SimulationError(
+                """
+                Failed while preparing to run the model. Both increment and timespan are set.
+
+                To continue either remove your `timespan` definition from your Model or remove the 
+                `increment` argument from this `model.run()` call.               
+                """
+            )
+
         try:
-            solver_results, rc = solver.run(model=self, t=t, increment=increment, timeout=timeout, **solver_args)
+            return solver.run(model=self, t=t, increment=increment, timeout=timeout, **solver_args)
         except Exception as e:
             # If user has specified the SSACSolver, but they don't actually have a g++ compiler,
             # This will throw an error and throw log. IF a user specifies cpp_support == True and don't have a compiler
@@ -988,27 +999,6 @@ class Model(SortableObject, Jsonify):
             raise SimulationError(
                 "argument 'solver={}' to run() failed.  Reason Given: {}".format(solver, e))
 
-        if rc == 33:
-            from gillespy2.core import log
-            log.warning('GillesPy2 simulation exceeded timeout.')
-
-        if hasattr(solver_results[0], 'shape'):
-            return solver_results
-
-        if len(solver_results) > 0:
-            results_list = []
-            for i in range(0, len(solver_results)):
-                temp = Trajectory(data=solver_results[i], model=self, solver_name=solver.name, rc=rc)
-                results_list.append(temp)
-
-            results = Results(results_list)
-            if show_labels == False:
-                results = results.to_array()
-            return results
-
-        else:
-            raise ValueError("number_of_trajectories must be non-negative and non-zero")
-
 
 class StochMLDocument():
     """ Serializiation and deserialization of a Model to/from
@@ -1166,7 +1156,7 @@ class StochMLDocument():
                 p = Parameter(name, expression=expr)
                 # Try to evaluate the expression in the empty namespace
                 # (if the expr is a scalar value)
-                p.evaluate()
+                p._evaluate()
                 model.add_parameter(p)
 
         # Create species
@@ -1264,7 +1254,7 @@ class StochMLDocument():
                         p = Parameter(name=generated_rate_name,
                                       expression=ratename)
                         # Try to evaluate the parameter to set its value
-                        p.evaluate()
+                        p._evaluate()
                         model.add_parameter(p)
                         reaction.marate = model.listOfParameters[
                             generated_rate_name]
