@@ -77,7 +77,10 @@ class Reaction(SortableObject, Jsonify):
         """
 
         # Metadata
-        self.name = name
+        if name in (None, ""):
+            self.name = f'rxn{uuid.uuid4()}'.replace('-', '_')
+        else:
+            self.name = name
         self.annotation = ""
 
         # We might use this flag in the future to automatically generate
@@ -163,6 +166,10 @@ class Reaction(SortableObject, Jsonify):
                 expr = ExpressionParser().visit(expr)
 
                 class ToString(ast.NodeVisitor):
+                    substitutions = {
+                        "ln": "log",
+                    }
+
                     def __init__(self):
                         self.string = ''
 
@@ -185,13 +192,16 @@ class Reaction(SortableObject, Jsonify):
                         self.generic_visit(node)
 
                     def visit_Call(self, node):
-                        self._string_changer(node.func.id + '(')
+                        func_name = ToString.substitutions.get(node.func.id) \
+                            if node.func.id in ToString.substitutions \
+                            else node.func.id
+                        self._string_changer(func_name + '(')
                         counter = 0
                         for arg in node.args:
-                            self.visit(arg)
-                            if counter == 0:
+                            if counter > 0:
                                 self._string_changer(',')
-                                counter += 1
+                            self.visit(arg)
+                            counter += 1
                         self._string_changer(')')
 
                     def visit_Add(self, node):
@@ -222,7 +232,9 @@ class Reaction(SortableObject, Jsonify):
                 newFunc = ToString()
                 newFunc.visit(expr)
                 return newFunc.string
+
             self.propensity_function = __customPropParser()
+            self.ode_propensity_function = self.propensity_function
 
     def __str__(self):
         self.verify()
@@ -295,8 +307,7 @@ class Reaction(SortableObject, Jsonify):
                 rname = r.name
             # Case 1: 2X -> Y
             if self.reactants[r] == 2:
-                propensity_function = (propensity_function +
-                                       "*" + rname + "*(" + rname + "-1)/vol")
+                propensity_function = "0.5 * {0} * {1} * ({1} - 1) / vol".format(propensity_function, rname)
                 ode_propensity_function += '*' + rname + '*' + rname
             else:
                 # Case 3: X1, X2 -> Y;
