@@ -39,25 +39,6 @@ unsigned int number_timesteps = 0;
 double end_time = 0;
 double tau_tol = 0.03;
 
-class PropensityFunction : public IPropensityFunction
-{
-public:
-	double TauEvaluate(unsigned int reaction_number, const std::vector<int> &S)
-	{
-		return map_propensity(reaction_number, S);
-	}
-
-	double evaluate(unsigned int reaction_number, unsigned int *state)
-	{
-		return 1.0;
-	}
-
-	double ODEEvaluate(int reaction_number, const std::vector<double> &S)
-	{
-		return 1.0;
-	}
-};
-
 int main(int argc, char* argv[]){
 	ArgParser parser = ArgParser(argc, argv);
 
@@ -73,14 +54,13 @@ int main(int argc, char* argv[]){
 	number_timesteps = parser.timesteps;
 	tau_tol = parser.tau_tol;
 
+	Reaction::load_parameters();
 	Model<unsigned int> model(species_names, species_populations, reaction_names);
 	add_reactions(model);
 
 	if(seed_time) {
 		random_seed = time(NULL);
 	}
-
-	IPropensityFunction *propFun = new PropensityFunction();
 	Simulation<unsigned int> simulation;
 
 	simulation.model = &model;
@@ -88,13 +68,11 @@ int main(int argc, char* argv[]){
 	simulation.random_seed = random_seed;
 	simulation.number_timesteps = number_timesteps;
 	simulation.number_trajectories = number_trajectories;
-	simulation.propensity_function = propFun;
+	simulation.output_interval = parser.output_interval;
 
 	init_simulation(&model, simulation);
 	tau_leaper(&simulation, tau_tol);
+	simulation.output_buffer_final(std::cout);
 
-	simulation.output_results_buffer(std :: cout);
-
-	delete propFun;
 	return 0;
 }
