@@ -19,14 +19,19 @@
 #include <vector>
 #include <string>
 #include <sstream>
+#include <cmath>
 
 #include "template.h"
 #include "model.h"
 #include "template_definitions.h"
 #include "template_defaults.h"
+#include "template_params.h"
 
 namespace Gillespy
 {
+	static double param_overrides[GPY_PARAMETER_NUM_VARIABLES];
+	static bool param_override_mask[GPY_PARAMETER_NUM_VARIABLES];
+
 	double populations[GPY_NUM_SPECIES] = GPY_INIT_POPULATIONS;
 	std::vector<double> species_populations(
 		populations,
@@ -44,7 +49,6 @@ namespace Gillespy
 		s_names + sizeof(s_names) / sizeof(std::string));
 
 	int reactions[GPY_NUM_REACTIONS][GPY_NUM_SPECIES] = GPY_REACTIONS;
-	// std::string r_names[GPY_NUM_REACTIONS] = GPY_REACTION_NAMES;
 	std::string r_names[GPY_NUM_REACTIONS] = 
 	{
 		#define REACTION_NAME(name) #name,
@@ -56,13 +60,35 @@ namespace Gillespy
 		r_names,
 		r_names + sizeof(r_names) / sizeof(std::string));
 
-	#define VARIABLE(name, value) static double (name) = (value);
-	#define CONSTANT(name, value) static const double (name) = (value);
-	GPY_PARAMETER_VALUES
-	#undef CONSTANT
-	#undef VARIABLE
+	double *get_variables(int *num_variables)
+	{
+		double *variables = new double[GPY_PARAMETER_NUM_VARIABLES];
 
-	double map_propensity(int reaction_id, const std::vector<int> &S)
+		#define CONSTANT(id, value)
+		#define VARIABLE(id, value) variables[id] = param_override_mask[id] \
+			? param_overrides[id] \
+			: value; (*num_variables)++;
+		GPY_PARAMETER_VALUES
+		#undef VARIABLE
+		#undef CONSTANT
+
+		return variables;
+	}
+
+	double *get_constants(int *num_constants)
+	{
+		double *constants = new double[GPY_PARAMETER_NUM_CONSTANTS];
+
+		#define VARIABLE(id, value)
+		#define CONSTANT(id, value) constants[id] = value; (*num_constants)++;
+		GPY_PARAMETER_VALUES
+		#undef CONSTANT
+		#undef VARIABLE
+
+		return constants;
+	}
+
+	double map_propensity(unsigned int reaction_id, unsigned int *S, double *P, const double *C)
 	{
 		switch (reaction_id)
 		{
@@ -75,7 +101,7 @@ namespace Gillespy
 		}
 	}
 
-	double map_propensity(int reaction_id, unsigned int *S)
+	double map_propensity(unsigned int reaction_id, int *S, double *P, const double *C)
 	{
 		switch (reaction_id)
 		{
@@ -88,7 +114,7 @@ namespace Gillespy
 		}
 	}
 
-	double map_ode_propensity(int reaction_id, const std::vector<double> &S)
+	double map_propensity(unsigned int reaction_id, double *S, double *P, const double *C)
 	{
 		switch (reaction_id)
 		{
@@ -103,8 +129,10 @@ namespace Gillespy
 
 	void map_variable_parameters(std::stringstream &stream)
 	{
-		#define VARIABLE(name, value) stream >> (name);
-		#define CONSTANT(name, value)
+		#define VARIABLE(id, value) \
+		stream >> param_overrides[id]; \
+		param_override_mask[id] = true;
+		#define CONSTANT(id, value)
 		GPY_PARAMETER_VALUES
 		#undef CONSTANT
 		#undef VARIABLE
@@ -142,4 +170,5 @@ namespace Gillespy
 
 	template void add_reactions<double>(Model<double> &model);
 	template void add_reactions<unsigned int>(Model<unsigned int> &model);
+	template void add_reactions<int>(Model<int> &model);
 }
