@@ -25,6 +25,7 @@ from gillespy2.solvers.utilities import Tau
 from gillespy2.solvers.utilities import solverutils as nputils
 from gillespy2.core import GillesPySolver, log, liveGraphing
 from gillespy2.core import ModelError, ExecutionError
+from gillespy2.core.results import Results
 
 class TauLeapingSolver(GillesPySolver):
     """
@@ -40,12 +41,13 @@ class TauLeapingSolver(GillesPySolver):
     pause_event = None
     result = None
 
-    def __init__(self, debug=False, profile=False):
+    def __init__(self, model=None, debug=False, profile=False):
         name = "TauLeapingSolver"
         rc = 0
         stop_event = None
         pause_event = None
         result = None
+        self.model = model
         self.debug = debug
         self.profile = profile
 
@@ -87,155 +89,164 @@ class TauLeapingSolver(GillesPySolver):
         return ('model', 't', 'number_of_trajectories', 'increment', 'seed', 'debug', 'profile','timeout', 'tau_tol')
 
     @classmethod
-    def run(self, model, t=20, number_of_trajectories=1, increment=0.05, seed=None,
+    def run(self, model=None, t=20, number_of_trajectories=1, increment=None, seed=None,
             debug=False, profile=False,  live_output=None, live_output_options={},
             timeout=None, resume=None, tau_tol=0.03, **kwargs):
-            """
-            Function calling simulation of the model.
-            This is typically called by the run function in GillesPy2 model objects
-            and will inherit those parameters which are passed with the model
-            as the arguments this run function.
+        """
+        Function calling simulation of the model.
+        This is typically called by the run function in GillesPy2 model objects
+        and will inherit those parameters which are passed with the model
+        as the arguments this run function.
 
-            :param model: GillesPy2 model object to simulate
-            :type model: gillespy2.Model
+        :param model: GillesPy2 model object to simulate
+        :type model: gillespy2.Model
 
-            :param t: Simulation run time
-            :type t: int
+        :param t: Simulation run time
+        :type t: int
 
-            :param number_of_trajectories: Number of trajectories to simulate
-            :type number_of_trajectories: int
+        :param number_of_trajectories: Number of trajectories to simulate
+        :type number_of_trajectories: int
 
-            :param increment: Save point increment for recording data
-            :type increment: float
+        :param increment: Save point increment for recording data
+        :type increment: float
 
-            :param seed: The random seed for the simulation. Optional, defaults to None
-            :type seed: int
+        :param seed: The random seed for the simulation. Optional, defaults to None
+        :type seed: int
 
-            :param debug: Set to True to provide additional debug information about the simulation
-            :type debug: bool
+        :param debug: Set to True to provide additional debug information about the simulation
+        :type debug: bool
 
-            :param profile: Set to True to provide information about step size (tau) taken at each step.
-            :type profile: bool
+        :param profile: Set to True to provide information about step size (tau) taken at each step.
+        :type profile: bool
 
-            :param live_output: The type of output to be displayed by solver. Can be "progress", "text", or "graph".
-            :type live_output: str
+        :param live_output: The type of output to be displayed by solver. Can be "progress", "text", or "graph".
+        :type live_output: str
 
-            :param live_output_options: COntains options for live_output. By default {"interval":1}. "interval"
-                specifies seconds between displaying. "clear_output" specifies if display should be refreshed with each
-                display.
-            :type live_output_options: dict
+        :param live_output_options: COntains options for live_output. By default {"interval":1}. "interval"
+            specifies seconds between displaying. "clear_output" specifies if display should be refreshed with each
+            display.
+        :type live_output_options: dict
 
-            :param timeout:
-            :param resume:
-            :param tau_tol:
-            :param kwargs:
+        :param timeout:
+        :param resume:
+        :param tau_tol:
+        :param kwargs:
 
-            :returns:
-            """
+        :returns:
+        """
 
-            if isinstance(self, type):
-                self = TauLeapingSolver(debug=debug, profile=profile)
+        if isinstance(self, type):
+            self = TauLeapingSolver(model=model, debug=debug, profile=profile)
+        if self.model is None:
+            if model is None:
+                raise SimulationError("A model is required to run the simulation.")
+            self.model = model
+        if model is not None and model.get_json_hash() != self.model.get_json_hash():
+            raise SimulationError("Model must equal TauLeapingSolver.model.")
+        self.model.resolve_parameters()
 
-            self.stop_event = Event()
-            self.pause_event = Event()
+        increment = self.get_increment(increment=increment)
 
-            if timeout is not None and timeout <= 0:
-                timeout = None
-            if len(kwargs) > 0:
-                for key in kwargs:
-                    log.warning('Unsupported keyword argument to {0} solver: {1}'.format(self.name, key))
+        self.stop_event = Event()
+        self.pause_event = Event()
 
-            # create numpy array for timeline
-            if resume is not None:
-                # start where we last left off if resuming a simulatio
-                lastT = resume['time'][-1]
-                step = lastT - resume['time'][-2]
-                timeline = np.arange(lastT, t+step, step)
-            else:
-                timeline = np.linspace(0, t, int(round(t / increment + 1)))
+        if timeout is not None and timeout <= 0:
+            timeout = None
+        if len(kwargs) > 0:
+            for key in kwargs:
+                log.warning('Unsupported keyword argument to {0} solver: {1}'.format(self.name, key))
 
-            species = list(model._listOfSpecies.keys())
-            trajectory_base, tmpSpecies = nputils.numpy_trajectory_base_initialization(model, number_of_trajectories,
-                                                                                       timeline, species, resume=resume)
+        # create numpy array for timeline
+        if resume is not None:
+            # start where we last left off if resuming a simulatio
+            lastT = resume['time'][-1]
+            step = lastT - resume['time'][-2]
+            timeline = np.arange(lastT, t+step, step)
+        else:
+            timeline = np.linspace(0, t, int(round(t / increment + 1)))
 
-            # total_time and curr_state are list of len 1 so that __run receives reference
-            if resume is not None:
-                total_time = [resume['time'][-1]]
-            else:
-                total_time = [0]
+        species = list(self.model._listOfSpecies.keys())
+        trajectory_base, tmpSpecies = nputils.numpy_trajectory_base_initialization(self.model, number_of_trajectories,
+                                                                                   timeline, species, resume=resume)
 
-            curr_state = [None]
-            live_grapher = [None]
+        # total_time and curr_state are list of len 1 so that __run receives reference
+        if resume is not None:
+            total_time = [resume['time'][-1]]
+        else:
+            total_time = [0]
 
-            sim_thread = Thread(target=self.___run, args=(model, curr_state, total_time, timeline, trajectory_base, tmpSpecies,
-                                                          live_grapher,), kwargs={'t': t,
-                                                                                  'number_of_trajectories':
-                                                                                      number_of_trajectories,
-                                                                                  'increment': increment, 'seed': seed,
-                                                                                  'debug': debug, 'resume': resume,
-                                                                                  'timeout': timeout, 'tau_tol': tau_tol
-                                                                                  })
-            try:
-                time = 0
-                sim_thread.start()
-                if live_output is not None:
-                    import gillespy2.core.liveGraphing
-                    live_output_options['type'] = live_output
-                    gillespy2.core.liveGraphing.valid_graph_params(
-                        live_output_options)
-                    if resume is not None:
-                        resumeTest = True  # If resuming, relay this information to live_grapher
-                    else:
-                        resumeTest = False
-                    live_grapher[
-                        0] = gillespy2.core.liveGraphing.LiveDisplayer(model,
-                                                                       timeline,
-                                                                       number_of_trajectories,
-                                                                       live_output_options,
-                                                                       resume=resumeTest)
-                    display_timer = gillespy2.core.liveGraphing.RepeatTimer(
-                        live_output_options['interval'],
-                        live_grapher[0].display, args=(curr_state,
-                                                       total_time,
-                                                       trajectory_base,
-                                                       live_output
-                                                       )
-                        )
-                    display_timer.start()
+        curr_state = [None]
+        live_grapher = [None]
 
-                if timeout is not None:
-                    while sim_thread.is_alive():
-                        sim_thread.join(.1)
-                        time += .1
-                        if time >= timeout:
-                            break
+        sim_thread = Thread(target=self.___run, args=(curr_state, total_time, timeline, trajectory_base, tmpSpecies,
+                                                      live_grapher,), kwargs={'t': t,
+                                                                              'number_of_trajectories':
+                                                                                  number_of_trajectories,
+                                                                              'increment': increment, 'seed': seed,
+                                                                              'debug': debug, 'resume': resume,
+                                                                              'timeout': timeout, 'tau_tol': tau_tol
+                                                                              })
+        try:
+            time = 0
+            sim_thread.start()
+            if live_output is not None:
+                import gillespy2.core.liveGraphing
+                live_output_options['type'] = live_output
+                gillespy2.core.liveGraphing.valid_graph_params(
+                    live_output_options)
+                if resume is not None:
+                    resumeTest = True  # If resuming, relay this information to live_grapher
                 else:
-                    while sim_thread.is_alive():
-                        sim_thread.join(.1)
+                    resumeTest = False
+                live_grapher[
+                    0] = gillespy2.core.liveGraphing.LiveDisplayer(self.model,
+                                                                   timeline,
+                                                                   number_of_trajectories,
+                                                                   live_output_options,
+                                                                   resume=resumeTest)
+                display_timer = gillespy2.core.liveGraphing.RepeatTimer(
+                    live_output_options['interval'],
+                    live_grapher[0].display, args=(curr_state,
+                                                   total_time,
+                                                   trajectory_base,
+                                                   live_output
+                                                   )
+                    )
+                display_timer.start()
 
-                if live_grapher[0] is not None:
-                    display_timer.cancel()
-                self.stop_event.set()
-                while self.result is None:
-                    pass
-            except KeyboardInterrupt:
-                if live_output:
-                    display_timer.pause = True
-                    display_timer.cancel()
-                self.pause_event.set()
-                while self.result is None:
-                    pass
-            if hasattr(self, 'has_raised_exception'):
-                raise self.has_raised_exception
+            if timeout is not None:
+                while sim_thread.is_alive():
+                    sim_thread.join(.1)
+                    time += .1
+                    if time >= timeout:
+                        break
+            else:
+                while sim_thread.is_alive():
+                    sim_thread.join(.1)
 
-            return self.result, self.rc
+            if live_grapher[0] is not None:
+                display_timer.cancel()
+            self.stop_event.set()
+            while self.result is None:
+                pass
+        except KeyboardInterrupt:
+            if live_output:
+                display_timer.pause = True
+                display_timer.cancel()
+            self.pause_event.set()
+            while self.result is None:
+                pass
+        if hasattr(self, 'has_raised_exception'):
+            raise self.has_raised_exception
 
-    def ___run(self, model, curr_state,total_time, timeline, trajectory_base, tmpSpecies, live_grapher, t=20,
+        return Results.build_from_solver_results(self, live_output_options)
+
+    def ___run(self, curr_state,total_time, timeline, trajectory_base, tmpSpecies, live_grapher, t=20,
                number_of_trajectories=1, increment=0.05, seed=None, debug=False, profile=False, show_labels=True,
                timeout=None, resume=None, tau_tol=0.03, **kwargs):
 
         try:
-            self.__run(model, curr_state, total_time, timeline, trajectory_base, tmpSpecies, live_grapher, t, number_of_trajectories,
+            self.__run(curr_state, total_time, timeline, trajectory_base, tmpSpecies, live_grapher, t, number_of_trajectories,
                        increment, seed, debug, profile, timeout, resume, tau_tol, **kwargs)
 
         except Exception as e:
@@ -243,7 +254,7 @@ class TauLeapingSolver(GillesPySolver):
             self.result = []
             return [], -1
 
-    def __run(self, model, curr_state, total_time, timeline, trajectory_base, tmpSpecies, live_grapher, t=20,
+    def __run(self, curr_state, total_time, timeline, trajectory_base, tmpSpecies, live_grapher, t=20,
               number_of_trajectories=1, increment=0.05, seed=None, debug=False, profile=False, timeout=None,
               resume=None, tau_tol=0.03, **kwargs):
 
@@ -251,7 +262,7 @@ class TauLeapingSolver(GillesPySolver):
         # how species and time are initialized to 0
         timeStopped = 0
         if resume is not None:
-            if resume[0].model != model:
+            if resume[0].model != self.model:
                 raise ModelError('When resuming, one must not alter the model being resumed.')
             if t < resume['time'][-1]:
                 raise ExecutionError(
@@ -261,7 +272,7 @@ class TauLeapingSolver(GillesPySolver):
             print("t = ", t)
             print("increment = ", increment)
 
-        species_mappings, species, parameter_mappings, number_species = nputils.numpy_initialization(model)
+        species_mappings, species, parameter_mappings, number_species = nputils.numpy_initialization(self.model)
 
         if seed is not None:
             if not isinstance(seed, int):
@@ -284,7 +295,7 @@ class TauLeapingSolver(GillesPySolver):
             if live_grapher[0] is not None:
                 live_grapher[0].increment_trajectory(trajectory_num)
 
-            start_state = [0] * (len(model.listOfReactions) + len(model.listOfRateRules))
+            start_state = [0] * (len(self.model.listOfReactions) + len(self.model.listOfRateRules))
             propensities = {}
             curr_state[0] = {}
 
@@ -295,35 +306,35 @@ class TauLeapingSolver(GillesPySolver):
                 save_time = 0
                 curr_time = [0]
 
-            curr_state[0]['vol'] = model.volume
+            curr_state[0]['vol'] = self.model.volume
             data = {'time': timeline}
             steps_taken = []
             steps_rejected = 0
             entry_count = 0
             trajectory = trajectory_base[trajectory_num]
 
-            HOR, reactants, mu_i, sigma_i, g_i, epsilon_i, critical_threshold = Tau.initialize(model, tau_tol)
+            HOR, reactants, mu_i, sigma_i, g_i, epsilon_i, critical_threshold = Tau.initialize(self.model, tau_tol)
 
             if resume is not None:
-                for spec in model.listOfSpecies:
+                for spec in self.model.listOfSpecies:
                     curr_state[0][spec] = tmpSpecies[spec]
             else:
-                for spec in model.listOfSpecies:
-                    curr_state[0][spec] = model.listOfSpecies[spec].initial_value
+                for spec in self.model.listOfSpecies:
+                    curr_state[0][spec] = self.model.listOfSpecies[spec].initial_value
 
-            for param in model.listOfParameters:
-                curr_state[0][param] = model.listOfParameters[param].value
+            for param in self.model.listOfParameters:
+                curr_state[0][param] = self.model.listOfParameters[param].value
 
-            for i, rxn in enumerate(model.listOfReactions):
+            for i, rxn in enumerate(self.model.listOfReactions):
                 # set reactions to uniform random number and add to start_state
                 start_state[i] = (math.log(random.uniform(0, 1)))
                 if debug:
                     print("Setting Random number ",
-                          start_state[i], " for ", model.listOfReactions[rxn].name)
+                          start_state[i], " for ", self.model.listOfReactions[rxn].name)
 
             compiled_propensities = {}
-            for i, r in enumerate(model.listOfReactions):
-                compiled_propensities[r] = compile(model.listOfReactions[r].propensity_function, '<string>', 'eval')
+            for i, r in enumerate(self.model.listOfReactions):
+                compiled_propensities[r] = compile(self.model.listOfReactions[r].propensity_function, '<string>', 'eval')
 
             timestep = 0
             
@@ -347,12 +358,12 @@ class TauLeapingSolver(GillesPySolver):
 
                     propensity_sum = 0
 
-                    for i, r in enumerate(model.listOfReactions):
+                    for i, r in enumerate(self.model.listOfReactions):
                         propensities[r] = eval(compiled_propensities[r], curr_state[0])
                         propensity_sum += propensities[r]
 
                     tau_args = [HOR, reactants, mu_i, sigma_i, g_i, epsilon_i, tau_tol, critical_threshold,
-                                model, propensities, curr_state[0], curr_time[0], save_time]
+                                self.model, propensities, curr_state[0], curr_time[0], save_time]
 
                     tau_step = Tau.select(*tau_args)
 
@@ -369,19 +380,19 @@ class TauLeapingSolver(GillesPySolver):
 
                         reactions, curr_state[0], curr_time[0] = self.__get_reactions(
                             tau_step, curr_state[0], curr_time[0], save_time,
-                            propensities, model.listOfReactions)
+                            propensities, self.model.listOfReactions)
 
                         # Update curr_state with the result of the SSA reaction that fired
                         species_modified = {}
-                        for i, rxn in enumerate(model.listOfReactions):
+                        for i, rxn in enumerate(self.model.listOfReactions):
                             if reactions[rxn] > 0:
-                                for reactant in model.listOfReactions[rxn].reactants:
+                                for reactant in self.model.listOfReactions[rxn].reactants:
                                     species_modified[reactant.name] = True
-                                    curr_state[0][reactant.name] -= model.listOfReactions[
+                                    curr_state[0][reactant.name] -= self.model.listOfReactions[
                                         rxn].reactants[reactant] * reactions[rxn]
-                                for product in model.listOfReactions[rxn].products:
+                                for product in self.model.listOfReactions[rxn].products:
                                     species_modified[product.name] = True
-                                    curr_state[0][product.name] += model.listOfReactions[
+                                    curr_state[0][product.name] += self.model.listOfReactions[
                                         rxn].products[product] * reactions[rxn]
                         neg_state = False
                         for spec in species_modified:
